package me.regexp;

import java.util.*;

public final class RECompiler
{
    private char[] instruction;
    private int lenInstruction;
    private String pattern;
    private int len;
    private int idx;
    private int parens;
    private int anInt386;
    private int anInt387;
    private static Hashtable hashPOSIX;
    
    public RECompiler() {
        super();
        this.instruction = new char[128];
        this.lenInstruction = 0;
    }
    
    private void ensure(final int n) {
        int length = this.instruction.length;
        if (this.lenInstruction + n >= length) {
            while (this.lenInstruction + n >= length) {
                length <<= 1;
            }
            final char[] aCharArray379 = new char[length];
            System.arraycopy(this.instruction, 0, aCharArray379, 0, this.lenInstruction);
            this.instruction = aCharArray379;
        }
    }
    
    private void emit(final char c) {
        this.ensure(1);
        this.instruction[this.lenInstruction++] = c;
    }
    
    private void nodeInsert(final char c, final int n) {
        this.ensure(3);
        System.arraycopy(this.instruction, n, this.instruction, n + 3, this.lenInstruction - n);
        this.instruction[n] = c;
        this.instruction[n + 1] = '\0';
        this.instruction[n + 2] = '\0';
        this.lenInstruction += 3;
    }
    
    private void setNextOfEnd(int n, int anInt380) {
        for (char c = this.instruction[n + 2]; c != '\0' && n < this.lenInstruction; n += c, c = this.instruction[n + 2]) {
            if (n == anInt380) {
                anInt380 = this.lenInstruction;
            }
        }
        if (n < this.lenInstruction) {
            final int n2 = anInt380 - n;
            if ((anInt380 = n2) != (short)n2) {
                throw new RESyntaxException("Exceeded short jump range.");
            }
            this.instruction[n + 2] = (char)anInt380;
        }
    }
    
    private int node(final char c, final int n) {
        this.ensure(3);
        this.instruction[this.lenInstruction] = c;
        this.instruction[this.lenInstruction + 1] = (char)n;
        this.instruction[this.lenInstruction + 2] = '\0';
        this.lenInstruction += 3;
        return this.lenInstruction - 3;
    }
    
    private static void internalError() {
        throw new Error("Internal error!");
    }
    
    private static void syntaxError(final String s) {
        throw new RESyntaxException(s);
    }
    
    private int escape() {
        if (this.pattern.charAt(this.idx) != '\\') {
            internalError();
        }
        if (this.idx + 1 == this.len) {
            syntaxError("Escape terminates string");
        }
        this.idx += 2;
        final char char1;
        switch (char1 = this.pattern.charAt(this.idx - 1)) {
            case 'B':
            case 'b':
                return 1048574;
            case 'D':
            case 'S':
            case 'W':
            case 'd':
            case 's':
            case 'w':
                return 1048573;
            case 'u':
            case 'x': {
                int n = (char1 == 'u') ? 4 : 2;
                int n2 = 0;
                while (this.idx < this.len && n-- > 0) {
                    final char char2;
                    if ((char2 = this.pattern.charAt(this.idx)) >= '0' && char2 <= '9') {
                        n2 = (n2 << 4) + char2 - 48;
                    }
                    else {
                        final char lowerCase;
                        if ((lowerCase = Character.toLowerCase(char2)) >= 'a' && lowerCase <= 'f') {
                            n2 = (n2 << 4) + (lowerCase - 'a') + 10;
                        }
                        else {
                            syntaxError("Expected " + n + " hexadecimal digits after \\" + char1);
                        }
                    }
                    ++this.idx;
                }
                return n2;
            }
            case 't':
                return 9;
            case 'n':
                return 10;
            case 'r':
                return 13;
            case 'f':
                return 12;
            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
                if ((this.idx < this.len && Character.isDigit(this.pattern.charAt(this.idx))) || char1 == '0') {
                    int n3 = char1 - '0';
                    if (this.idx < this.len && Character.isDigit(this.pattern.charAt(this.idx))) {
                        n3 = (n3 << 3) + (this.pattern.charAt(this.idx++) - '0');
                        if (this.idx < this.len && Character.isDigit(this.pattern.charAt(this.idx))) {
                            n3 = (n3 << 3) + (this.pattern.charAt(this.idx++) - '0');
                        }
                    }
                    return n3;
                }
                return 1048575;
            default:
                return char1;
        }
    }
    
    private int atom() {
        final int ret = this.node('A', 0);
        int n = 0;
    Label_0360:
        while (this.idx < this.len) {
            if (this.idx + 1 < this.len) {
                char c = this.pattern.charAt(this.idx + 1);
                if (this.pattern.charAt(this.idx) == '\\') {
                    final int anInt384 = this.idx;
                    this.escape();
                    if (this.idx < this.len) {
                        c = this.pattern.charAt(this.idx);
                    }
                    this.idx = anInt384;
                }
                switch (c) {
                    case 42:
                    case 43:
                    case 63:
                    case 123:
                        if (n == 0) {
                            break;
                        }
                        break Label_0360;
                }
            }
            switch (this.pattern.charAt(this.idx)) {
                case '$':
                case '(':
                case ')':
                case '.':
                case '[':
                case ']':
                case '^':
                case '|':
                    break Label_0360;
                case '*':
                case '+':
                case '?':
                case '{':
                    if (n == 0) {
                        syntaxError("Missing operand to closure");
                        break Label_0360;
                    }
                    break Label_0360;
                case '\\': {
                    final int anInt385 = this.idx;
                    final int method318;
                    if (((method318 = this.escape()) & 0xFFFF0) == 0xFFFF0) {
                        this.idx = anInt385;
                        break Label_0360;
                    }
                    this.emit((char)method318);
                    ++n;
                    continue;
                }
                default:
                    this.emit(this.pattern.charAt(this.idx++));
                    ++n;
                    continue;
            }
        }
        if (n == 0) {
            internalError();
        }
        this.instruction[ret + 1] = (char)n;
        return ret;
    }
    
    private int terminal(final int[] array) {
        Label_1343: {
            switch (this.pattern.charAt(this.idx)) {
                case '$':
                case '.':
                case '^':
                    return this.node(this.pattern.charAt(this.idx++), 0);
                case '[': {
                    if (this.pattern.charAt(this.idx) != '[') {
                        internalError();
                    }
                    if (this.idx + 1 >= this.len || this.pattern.charAt(++this.idx) == ']') {
                        syntaxError("Empty or unterminated class");
                    }
                    if (this.idx < this.len && this.pattern.charAt(this.idx) == ':') {
                        ++this.idx;
                        final int anInt384 = this.idx;
                        while (this.idx < this.len && this.pattern.charAt(this.idx) >= 'a' && this.pattern.charAt(this.idx) <= 'z') {
                            ++this.idx;
                        }
                        if (this.idx + 1 < this.len && this.pattern.charAt(this.idx) == ':' && this.pattern.charAt(this.idx + 1) == ']') {
                            final String substring = this.pattern.substring(anInt384, this.idx);
                            final Character c;
                            if ((c = (Character) RECompiler.hashPOSIX.get(substring)) != null) {
                                this.idx += 2;
                                return this.node('P', c);
                            }
                            syntaxError("Invalid POSIX character class '" + substring + "'");
                        }
                        syntaxError("Invalid POSIX character class syntax");
                    }
                    final int method317 = this.node('[', 0);
                    char char1 = '\uffff';
                    boolean b = true;
                    int n = 0;
                    final int anInt385 = this.idx;
                    char c2 = '\0';
                    final RERange class106 = new RERange();
                    while (this.idx < this.len && this.pattern.charAt(this.idx) != ']') {
                        Label_1037: {
                            switch (this.pattern.charAt(this.idx)) {
                                case '^':
                                    b = !b;
                                    if (this.idx == anInt385) {
                                        class106.include(0, 65535, true);
                                    }
                                    ++this.idx;
                                    continue;
                                case '\\': {
                                    final int method318;
                                    switch (method318 = this.escape()) {
                                        case 1048574:
                                        case 1048575:
                                            syntaxError("Bad character class");
                                        case 1048573:
                                            if (n != 0) {
                                                syntaxError("Bad character class");
                                            }
                                            switch (this.pattern.charAt(this.idx - 1)) {
                                                case 'S':
                                                    class106.include(0, 7, b);
                                                    class106.include('\u000b', b);
                                                    class106.include(14, 31, b);
                                                    class106.include(33, 65535, b);
                                                    break;
                                                case 'W':
                                                    class106.include(0, 47, b);
                                                    class106.include(58, 64, b);
                                                    class106.include(91, 94, b);
                                                    class106.include('`', b);
                                                    class106.include(123, 65535, b);
                                                    break;
                                                case 'D':
                                                    class106.include(0, 47, b);
                                                    class106.include(58, 65535, b);
                                                    break;
                                                case 's':
                                                    class106.include('\t', b);
                                                    class106.include('\r', b);
                                                    class106.include('\f', b);
                                                    class106.include('\n', b);
                                                    class106.include('\b', b);
                                                    class106.include(' ', b);
                                                    break;
                                                case 'w':
                                                    class106.include(97, 122, b);
                                                    class106.include(65, 90, b);
                                                    class106.include('_', b);
                                                case 'd':
                                                    class106.include(48, 57, b);
                                                    break;
                                            }
                                            char1 = '\uffff';
                                            continue;
                                        default:
                                            char1 = (char)method318;
                                            break Label_1037;
                                    }
                                }
                                case '-':
                                    if (n != 0) {
                                        syntaxError("Bad class range");
                                    }
                                    n = 1;
                                    c2 = ((char1 == '\uffff') ? '\0' : char1);
                                    if (this.idx + 1 < this.len && this.pattern.charAt(++this.idx) == ']') {
                                        char1 = '\uffff';
                                        break;
                                    }
                                    continue;
                                default:
                                    char1 = this.pattern.charAt(this.idx++);
                                    break;
                            }
                        }
                        if (n != 0) {
                            if (c2 >= char1) {
                                syntaxError("Bad character class");
                            }
                            class106.include(c2, char1, b);
                            char1 = '\uffff';
                            n = 0;
                        }
                        else {
                            if (this.idx < this.len && this.pattern.charAt(this.idx) == '-') {
                                continue;
                            }
                            class106.include(char1, b);
                        }
                    }
                    if (this.idx == this.len) {
                        syntaxError("Unterminated character class");
                    }
                    ++this.idx;
                    this.instruction[method317 + 1] = (char)class106.num;
                    for (int i = 0; i < class106.num; ++i) {
                        this.emit((char)class106.minRange[i]);
                        this.emit((char)class106.maxRange[i]);
                    }
                    return method317;
                }
                case '(':
                    return this.expr(array);
                case ')':
                    syntaxError("Unexpected close paren");
                case '|':
                    internalError();
                case ']':
                    syntaxError("Mismatched class");
                case '\0':
                    syntaxError("Unexpected end of input");
                case '*':
                case '+':
                case '?':
                case '{':
                    syntaxError("Missing operand to closure");
                case '\\': {
                    final int anInt386 = this.idx;
                    switch (this.escape()) {
                        case 1048573:
                        case 1048574: {
                            final int n2 = 0;
                            array[n2] &= 0xFFFFFFFE;
                            return this.node('\\', this.pattern.charAt(this.idx - 1));
                        }
                        case 1048575: {
                            final char c3 = (char)(this.pattern.charAt(this.idx - 1) - '0');
                            if (this.parens <= c3) {
                                syntaxError("Bad backreference");
                            }
                            final int n3 = 0;
                            array[n3] |= 0x1;
                            return this.node('#', c3);
                        }
                        default: {
                            this.idx = anInt386;
                            final int n4 = 0;
                            array[n4] &= 0xFFFFFFFE;
                            break Label_1343;
                        }
                    }
                }
            }
        }
        final int n5 = 0;
        array[n5] &= 0xFFFFFFFE;
        return this.atom();
    }
    
    private int closure(int[] array) {
        final int anInt384 = this.idx;
        final int[] array2 = { 0 };
        final int method321 = this.terminal(array2);
        final int[] array3 = array;
        final int n = 0;
        array3[n] |= array2[0];
        if (this.idx >= this.len) {
            return method321;
        }
        boolean b = true;
        final char char1;
        switch (char1 = this.pattern.charAt(this.idx)) {
            case '*':
            case '?': {
                final int[] array4 = array;
                final int n2 = 0;
                array4[n2] |= 0x1;
            }
            case '+':
                ++this.idx;
            case '{': {
                final char c;
                if ((c = this.instruction[method321]) == '^' || c == '$') {
                    syntaxError("Bad closure operand");
                }
                if ((array2[0] & 0x1) != 0x0) {
                    syntaxError("Closure operand can't be nullable");
                    break;
                }
                break;
            }
        }
        if (this.idx < this.len && this.pattern.charAt(this.idx) == '?') {
            ++this.idx;
            b = false;
        }
        if (b) {
            switch (char1) {
                case 123: {
                    array = (int[])(Object)this;
                    if (this.idx >= ((RECompiler)(Object)array).len || ((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx++) != '{') {
                        internalError();
                    }
                    if (((RECompiler)(Object)array).idx >= ((RECompiler)(Object)array).len || !Character.isDigit(((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx))) {
                        syntaxError("Expected digit");
                    }
                    final StringBuffer sb = new StringBuffer();
                    while (((RECompiler)(Object)array).idx < ((RECompiler)(Object)array).len && Character.isDigit(((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx))) {
                        sb.append(((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx++));
                    }
                    try {
                        ((RECompiler)(Object)array).anInt386 = Integer.parseInt(sb.toString());
                    }
                    catch (NumberFormatException ex) {
                        syntaxError("Expected valid number");
                    }
                    if (((RECompiler)(Object)array).idx >= ((RECompiler)(Object)array).len) {
                        syntaxError("Expected comma or right bracket");
                    }
                    if (((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx) == '}') {
                        final int[] array5 = array;
                        ++((RECompiler)(Object)array5).idx;
                        ((RECompiler)(Object)array).anInt387 = 0;
                    }
                    else {
                        if (((RECompiler)(Object)array).idx >= ((RECompiler)(Object)array).len || ((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx++) != ',') {
                            syntaxError("Expected comma");
                        }
                        if (((RECompiler)(Object)array).idx >= ((RECompiler)(Object)array).len) {
                            syntaxError("Expected comma or right bracket");
                        }
                        if (((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx) == '}') {
                            final int[] array6 = array;
                            ++((RECompiler)(Object)array6).idx;
                            ((RECompiler)(Object)array).anInt387 = -1;
                        }
                        else {
                            if (((RECompiler)(Object)array).idx >= ((RECompiler)(Object)array).len || !Character.isDigit(((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx))) {
                                syntaxError("Expected digit");
                            }
                            sb.setLength(0);
                            while (((RECompiler)(Object)array).idx < ((RECompiler)(Object)array).len && Character.isDigit(((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx))) {
                                sb.append(((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx++));
                            }
                            try {
                                ((RECompiler)(Object)array).anInt387 = Integer.parseInt(sb.toString()) - ((RECompiler)(Object)array).anInt386;
                            }
                            catch (NumberFormatException ex2) {
                                syntaxError("Expected valid number");
                            }
                            if (((RECompiler)(Object)array).anInt387 < 0) {
                                syntaxError("Bad range");
                            }
                            if (((RECompiler)(Object)array).idx >= ((RECompiler)(Object)array).len || ((RECompiler)(Object)array).pattern.charAt(((RECompiler)(Object)array).idx++) != '}') {
                                syntaxError("Missing close brace");
                            }
                        }
                    }
                    final int anInt385 = this.idx;
                    final int anInt386 = this.anInt386;
                    final int anInt387 = this.anInt387;
                    int method322 = method321;
                    for (int i = 0; i < anInt386; ++i) {
                        this.idx = anInt384;
                        this.setNextOfEnd(method322, method322 = this.terminal(array2));
                    }
                    if (anInt387 == -1) {
                        this.idx = anInt385;
                        this.nodeInsert('*', method322);
                        this.setNextOfEnd(method322 + 3, method322);
                        break;
                    }
                    if (anInt387 > 0) {
                        final int[] array7 = new int[anInt387 + 1];
                        this.nodeInsert('?', method322);
                        array7[0] = method322;
                        for (int j = 1; j < anInt387; ++j) {
                            array7[j] = this.node('?', 0);
                            this.idx = anInt384;
                            this.terminal(array2);
                        }
                        final int[] array8 = array7;
                        final int n3 = anInt387;
                        final int method323 = this.node('N', 0);
                        array8[n3] = method323;
                        final int n4 = method323;
                        for (int k = 0; k < anInt387; ++k) {
                            this.setNextOfEnd(array7[k], n4);
                            this.setNextOfEnd(array7[k] + 3, array7[k + 1]);
                        }
                    }
                    else {
                        this.lenInstruction = method322;
                        this.node('N', 0);
                    }
                    this.idx = anInt385;
                    break;
                }
                case 63: {
                    this.nodeInsert('?', method321);
                    final int method324 = this.node('N', 0);
                    this.setNextOfEnd(method321, method324);
                    this.setNextOfEnd(method321 + 3, method324);
                    break;
                }
                case 42:
                    this.nodeInsert('*', method321);
                    this.setNextOfEnd(method321 + 3, method321);
                    break;
                case 43: {
                    this.nodeInsert('C', method321);
                    final int method325 = this.node('+', 0);
                    this.setNextOfEnd(method321 + 3, method325);
                    this.setNextOfEnd(method325, method321);
                    break;
                }
            }
        }
        else {
            switch (char1) {
                case 63: {
                    this.nodeInsert('/', method321);
                    final int method326 = this.node('N', 0);
                    this.setNextOfEnd(method321, method326);
                    this.setNextOfEnd(method321 + 3, method326);
                    break;
                }
                case 42:
                    this.nodeInsert('8', method321);
                    this.setNextOfEnd(method321 + 3, method321);
                    break;
                case 43: {
                    this.nodeInsert('C', method321);
                    final int method327 = this.node('=', 0);
                    this.setNextOfEnd(method327, method321);
                    this.setNextOfEnd(method321 + 3, method327);
                    break;
                }
            }
        }
        return method321;
    }
    
    private int branch(final int[] array) {
        int method317 = -1;
        int n = -1;
        final int[] array2 = { 0 };
        boolean b = true;
        while (this.idx < this.len && this.pattern.charAt(this.idx) != '|' && this.pattern.charAt(this.idx) != ')') {
            array2[0] = 0;
            final int method318 = this.closure(array2);
            if (array2[0] == 0) {
                b = false;
            }
            if (n != -1) {
                this.setNextOfEnd(n, method318);
            }
            n = method318;
            if (method317 == -1) {
                method317 = method318;
            }
        }
        if (method317 == -1) {
            method317 = this.node('N', 0);
        }
        if (b) {
            final int n2 = 0;
            array[n2] |= 0x1;
        }
        return method317;
    }
    
    private int expr(final int[] array) {
        int n = -1;
        int n2 = -1;
        final int anInt385 = this.parens;
        if ((array[0] & 0x2) == 0x0 && this.pattern.charAt(this.idx) == '(') {
            if (this.idx + 2 < this.len && this.pattern.charAt(this.idx + 1) == '?' && this.pattern.charAt(this.idx + 2) == ':') {
                n = 2;
                this.idx += 3;
                n2 = this.node('<', 0);
            }
            else {
                n = 1;
                ++this.idx;
                n2 = this.node('(', this.parens++);
            }
        }
        final int n3 = 0;
        array[n3] &= 0xFFFFFFFD;
        int n4 = 0;
        int n5 = this.branch(array);
        if (n2 == -1) {
            n2 = n5;
        }
        else {
            this.setNextOfEnd(n2, n5);
        }
        while (this.idx < this.len && this.pattern.charAt(this.idx) == '|') {
            if (n4 == 0) {
                this.nodeInsert('|', n5);
                n4 = 1;
            }
            ++this.idx;
            this.setNextOfEnd(n5, n5 = this.node('|', 0));
            this.branch(array);
        }
        int n6;
        if (n > 0) {
            if (this.idx < this.len && this.pattern.charAt(this.idx) == ')') {
                ++this.idx;
            }
            else {
                syntaxError("Missing close paren");
            }
            if (n == 1) {
                n6 = this.node(')', anInt385);
            }
            else {
                n6 = this.node('>', 0);
            }
        }
        else {
            n6 = this.node('E', 0);
        }
        this.setNextOfEnd(n2, n6);
        int n7 = n2;
        for (char c = this.instruction[n7 + '\u0002']; c != '\0' && n7 < this.lenInstruction; c = this.instruction[n7 + '\u0002'], n7 += c) {
            if (this.instruction[n7] == '|') {
                this.setNextOfEnd(n7 + 3, n6);
            }
        }
        return n2;
    }
    
    public final REProgram compile(final String aString381) {
        this.pattern = aString381;
        this.len = aString381.length();
        this.idx = 0;
        this.lenInstruction = 0;
        this.parens = 1;
        this.expr(new int[] { 2 });
        if (this.idx != this.len) {
            if (aString381.charAt(this.idx) == ')') {
                syntaxError("Unmatched close paren");
            }
            syntaxError("Unexpected input remains");
        }
        final char[] array = new char[this.lenInstruction];
        System.arraycopy(this.instruction, 0, array, 0, this.lenInstruction);
        return new REProgram(this.parens, array);
    }
    
    static {
        (RECompiler.hashPOSIX = new Hashtable()).put("alnum", new Character('w'));
        RECompiler.hashPOSIX.put("alpha", new Character('a'));
        RECompiler.hashPOSIX.put("blank", new Character('b'));
        RECompiler.hashPOSIX.put("cntrl", new Character('c'));
        RECompiler.hashPOSIX.put("digit", new Character('d'));
        RECompiler.hashPOSIX.put("graph", new Character('g'));
        RECompiler.hashPOSIX.put("lower", new Character('l'));
        RECompiler.hashPOSIX.put("print", new Character('p'));
        RECompiler.hashPOSIX.put("punct", new Character('!'));
        RECompiler.hashPOSIX.put("space", new Character('s'));
        RECompiler.hashPOSIX.put("upper", new Character('u'));
        RECompiler.hashPOSIX.put("xdigit", new Character('x'));
        RECompiler.hashPOSIX.put("javastart", new Character('j'));
        RECompiler.hashPOSIX.put("javapart", new Character('k'));
    }
}
