package me.regexp;

public final class REProgram
{
    static final int OPT_HASBACKREFS = 1;
    static final int OPT_HASBOL      = 2;

    char[] instruction;
    char[] prefix;
    int flags;
    int maxParens;
    
    public REProgram(final int anInt440, final char[] array) {
        this(array, array.length);
        this.maxParens = anInt440;
    }
    
    private REProgram(final char[] array, final int n) {
        super();
        this.maxParens = -1;
        this.setInstructions(array, n);
    }
    
    private void setInstructions(final char[] instruction, final int lenInstruction) {
        // Save reference to instruction array
        this.instruction = instruction;

        // Initialize other program-related variables
        this.flags = 0;
        this.prefix = null;

        // Try various compile-time optimizations if there's a program
        if (instruction != null && lenInstruction != 0)
        {
            // If the first node is a branch
            if (lenInstruction >= RE.nodeSize && instruction[0 + RE.offsetOpcode] == RE.OP_BRANCH)
            {
                // to the end node
                int next = (short) instruction[0 + RE.offsetNext];
                if (instruction[next + RE.offsetOpcode] == RE.OP_END && lenInstruction >= (RE.nodeSize * 2))
                {
                    final char nextOp = instruction[RE.nodeSize + RE.offsetOpcode];
                    // the branch starts with an atom
                    if (nextOp == RE.OP_ATOM)
                    {
                        // then get that atom as an prefix because there's no other choice
                        int lenAtom = instruction[RE.nodeSize + RE.offsetOpdata];
                        this.prefix = new char[lenAtom];
                        System.arraycopy(instruction, RE.nodeSize * 2, prefix, 0, lenAtom);
                    }
                    // the branch starts with a BOL
                    else if (nextOp == RE.OP_BOL)
                    {
                        // then set the flag indicating that BOL is present
                        this.flags |= OPT_HASBOL;
                    }
                }
            }

            BackrefScanLoop:

            // Check for backreferences
            for (int i = 0; i < lenInstruction; i += RE.nodeSize)
            {
                switch (instruction[i + RE.offsetOpcode])
                {
                    case RE.OP_ANYOF:
                        i += (instruction[i + RE.offsetOpdata] * 2);
                        break;

                    case RE.OP_ATOM:
                        i += instruction[i + RE.offsetOpdata];
                        break;

                    case RE.OP_BACKREF:
                        flags |= OPT_HASBACKREFS;
                        break BackrefScanLoop;
                }
            }
        }
    }
}
